@@ -1,5 +1,5 @@
 /*
- Copyright (c) 2006 Trevor Williams
+ Copyright (c) 2006-2009 Trevor Williams
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by the Free Software
@@ -15,13 +15,13 @@
 
 /*!
  \file     static.c
- \author   Trevor Williams  (trevorw@charter.net)
+ \author   Trevor Williams  (phase1geo@gmail.com)
  \date     10/02/2002
  
  \par
- To accommodate the need for parameters (variables) in static expressions, the static_expr
+ To accommodate the need for parameters/genvars (variables) in static expressions, the static_expr
  structure and supporting code was created to maintain the efficiency of static expressions
- that consist of known values while being able to keep track of parameter uses in
+ that consist of known values while being able to keep track of parameter/genvar uses in
  static expressions.
  
  \par
@@ -30,22 +30,22 @@
  it is assumed that the static_expr structure contains a valid, known value that can be
  used in immediate computations.  If the expression pointer is not NULL, it is assumed
  that the static_expr structure contains an expression tree that needs to be evaluated at
- a later time (when parameters are being elaborated).
+ a later time (when parameters/genvars are being elaborated).
  
  \par
  When a static expression is being parsed and a static value (integer value) is encountered,
  a new static_expr is allocated from heap memory and the number field is assigned to this
  integer value.  The new static_expr structure is then passed up the tree to be used in
  further calculations, if necessary.  If a static expression is being parsed and an identifier
- (parameter) is encountered, an expression is created with an operation type of EXP_OP_SIG
- to indicate that a parameter is required during elaboration.  The name of the necessary
- parameter is bound to the newly created expression.
+ (parameter/genvar) is encountered, an expression is created with an operation type of EXP_OP_SIG
+ to indicate that a parameter/genvar is required during elaboration.  The name of the necessary
+ parameter/genvar is bound to the newly created expression.
  
  \par
  Using this strategy for handling static expressions, it becomes evident that we retain
  the efficiency of calculating static expression that consists entirely of known values (the
  only overhead is the allocation/deallocation of a static_expr structure from the heap).
- If a parameter is found during the parse stage, more effort is required to calculate the
+ If a parameter/genvar is found during the parse stage, more effort is required to calculate the
  static_expr, but this is considered necessary in the larger scope of things so we will not
  concern ourselves with this overhead (which is fairly minimal anyways).
 */
@@ -75,6 +75,8 @@ extern int        curr_expr_id;
 
  \return Returns pointer to new static_expr structure.
 
+ \throws anonymous expression_create expression_create
+
  Used by the parser to calculate a new static_expr structure based on the
  unary operation encountered while parsing.  Based on the operation type
  specified in the argument list, performs unary operation (if operand is
@@ -84,11 +86,17 @@ extern int        curr_expr_id;
  expression, create an expression for the specified operation type and store
  this expression in the original expression pointer field.
 */
-static_expr* static_expr_gen_unary( static_expr* stexp, int op, int line, int first, int last ) {
+static_expr* static_expr_gen_unary(
+  static_expr* stexp,  /*!< Pointer to static expression */
+  exp_op_type  op,     /*!< Unary static expression operation */
+  int          line,   /*!< Line number that this expression was found on in file */
+  int          first,  /*!< Column index of first character in this expression */
+  int          last    /*!< Column index of last character in this expression */
+) { PROFILE(STATIC_EXPR_GEN_UNARY);
 
-  expression* tmpexp;  /* Container for newly created expression */
-  int uop;             /* Temporary bit holder */
-  int i;               /* Loop iterator */
+  expression*  tmpexp;  /* Container for newly created expression */
+  int          uop;     /* Temporary bit holder */
+  unsigned int i;       /* Loop iterator */
 
   if( stexp != NULL ) {
 
@@ -114,11 +122,13 @@ static_expr* static_expr_gen_unary( static_expr* stexp, int op, int line, int fi
           for( i=1; i<(SIZEOF_INT * 8); i++ ) {
             switch( op ) {
               case EXP_OP_UNAND :
+              /*@-shiftimplementation@*/
               case EXP_OP_UAND  :  uop = uop & ((stexp->num >> i) & 0x1);  break;
               case EXP_OP_UNOR  :
               case EXP_OP_UOR   :  uop = uop | ((stexp->num >> i) & 0x1);  break;
               case EXP_OP_UNXOR :
               case EXP_OP_UXOR  :  uop = uop ^ ((stexp->num >> i) & 0x1);  break;
+              /*@=shiftimplementation@*/
               default           :  break;
             }
           }
@@ -140,12 +150,15 @@ static_expr* static_expr_gen_unary( static_expr* stexp, int op, int line, int fi
         case EXP_OP_PASSIGN :
           tmpexp = expression_create( NULL, NULL, EXP_OP_STATIC, FALSE, curr_expr_id, line, first, last, FALSE );
           curr_expr_id++;
-          vector_init( tmpexp->value, (vec_data*)malloc_safe( (sizeof( vec_data ) * 32), __FILE__, __LINE__ ), 32 );
-          vector_from_int( tmpexp->value, stexp->num );
-        
+          {
+            vector* vec = vector_create( 32, VTYPE_EXP, VDATA_UL, TRUE );
+            vector_dealloc( tmpexp->value );
+            tmpexp->value = vec;
+          }
+          (void)vector_from_int( tmpexp->value, stexp->num );
+
           stexp->exp = expression_create( tmpexp, NULL, op, FALSE, curr_expr_id, line, first, last, FALSE );
           curr_expr_id++;
-          tmpexp->parent->expr = stexp->exp;
           break;
         default :  break;
       }
@@ -154,28 +167,23 @@ static_expr* static_expr_gen_unary( static_expr* stexp, int op, int line, int fi
 
       tmpexp = expression_create( stexp->exp, NULL, op, FALSE, curr_expr_id, line, first, last, FALSE );
       curr_expr_id++;
-      stexp->exp->parent->expr = tmpexp;
       stexp->exp = tmpexp;
 
     }
 
   }
 
+  PROFILE_END;
+
   return( stexp );
 
 }
 
 /*!
- \param right      Pointer to right static expression.
- \param left       Pointer to left static expression.
- \param op         Static expression operation.
- \param line       Line number that static expression operation found on.
- \param first      Column index of first character in expression.
- \param last       Column index of last character in expression.
- \param func_name  Name of function to call (only valid when op == EXP_OP_FUNC_CALL)
-
  \return Returns pointer to new static_expr structure.
 
+ \throws anonymous expression_create expression_create expression_create expression_create expression_create expression_create
+
  Used by the parser to calculate a new static_expr structure based on the
  operation encountered while parsing.  Based on the operation type
  specified in the argument list, performs unary operation (if both operands
@@ -186,7 +194,15 @@ static_expr* static_expr_gen_unary( static_expr* stexp, int op, int line, int fi
  consisting of those two expressions and specified operator.  Store the newly create
  expression in the original right static_expr and deallocate the left static_expr.
 */
-static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int line, int first, int last, char* func_name ) {
+static_expr* static_expr_gen(
+  static_expr* right,     /*!< Pointer to right static expression */
+  static_expr* left,      /*!< Pointer to left static expression */
+  int          op,        /*!< Static expression operation */
+  int          line,      /*!< Line number that static expression operation found on */
+  int          first,     /*!< Column index of first character in expression */
+  int          last,      /*!< Column index of last character in expression */
+  char*        func_name  /*!< Name of function to call (only valid when op == EXP_OP_FUNC_CALL) */
+) { PROFILE(STATIC_EXPR_GEN);
 
   expression* tmpexp;     /* Temporary expression for holding newly created parent expression */
   int         i;          /* Loop iterator */
@@ -195,7 +211,10 @@ static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int
   assert( (op == EXP_OP_XOR)    || (op == EXP_OP_MULTIPLY) || (op == EXP_OP_DIVIDE) || (op == EXP_OP_MOD)       ||
           (op == EXP_OP_ADD)    || (op == EXP_OP_SUBTRACT) || (op == EXP_OP_AND)    || (op == EXP_OP_OR)        ||
           (op == EXP_OP_NOR)    || (op == EXP_OP_NAND)     || (op == EXP_OP_NXOR)   || (op == EXP_OP_EXPONENT)  ||
-          (op == EXP_OP_LSHIFT) || (op == EXP_OP_RSHIFT)   || (op == EXP_OP_LIST)   || (op == EXP_OP_FUNC_CALL) );
+          (op == EXP_OP_LSHIFT) || (op == EXP_OP_RSHIFT)   || (op == EXP_OP_LIST)   || (op == EXP_OP_FUNC_CALL) ||
+          (op == EXP_OP_GE)     || (op == EXP_OP_LE)       || (op == EXP_OP_EQ)     || (op == EXP_OP_GT)        ||
+          (op == EXP_OP_LT)     || (op == EXP_OP_SBIT_SEL) || (op == EXP_OP_LAND)   || (op == EXP_OP_LOR)       ||
+          (op == EXP_OP_NE) );
 
   if( (right != NULL) && (left != NULL) ) {
 
@@ -221,8 +240,18 @@ static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int
           case EXP_OP_NOR      :  right->num = ~(left->num | right->num);  break;
           case EXP_OP_NAND     :  right->num = ~(left->num & right->num);  break;
           case EXP_OP_NXOR     :  right->num = ~(left->num ^ right->num);  break;
+          /*@-shiftnegative -shiftimplementation@*/
           case EXP_OP_LSHIFT   :  right->num = left->num << right->num;    break;
           case EXP_OP_RSHIFT   :  right->num = left->num >> right->num;    break;
+          /*@=shiftnegative =shiftimplementation@*/
+          case EXP_OP_GE       :  right->num = (left->num >= right->num) ? 1 : 0;  break;
+          case EXP_OP_LE       :  right->num = (left->num <= right->num) ? 1 : 0;  break;
+          case EXP_OP_EQ       :  right->num = (left->num == right->num) ? 1 : 0;  break;
+          case EXP_OP_NE       :  right->num = (left->num != right->num) ? 1 : 0;  break;
+          case EXP_OP_GT       :  right->num = (left->num > right->num)  ? 1 : 0;  break;
+          case EXP_OP_LT       :  right->num = (left->num < right->num)  ? 1 : 0;  break;
+          case EXP_OP_LAND     :  right->num = (left->num && right->num) ? 1 : 0;  break;
+          case EXP_OP_LOR      :  right->num = (left->num || right->num) ? 1 : 0;  break;
           default              :  break;
         }
 
@@ -230,13 +259,15 @@ static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int
 
         right->exp = expression_create( NULL, NULL, EXP_OP_STATIC, FALSE, curr_expr_id, line, first, last, FALSE );
         curr_expr_id++;
-        vector_init( right->exp->value, (vec_data*)malloc_safe( (sizeof( vec_data ) * 32), __FILE__, __LINE__ ), 32 );  
-        vector_from_int( right->exp->value, right->num );
+        {
+          vector* vec = vector_create( 32, VTYPE_EXP, VDATA_UL, TRUE );
+          vector_dealloc( right->exp->value );
+          right->exp->value = vec;
+        }
+        (void)vector_from_int( right->exp->value, right->num );
 
         tmpexp = expression_create( right->exp, left->exp, op, FALSE, curr_expr_id, line, first, last, FALSE );
         curr_expr_id++;
-        right->exp->parent->expr = tmpexp;
-        left->exp->parent->expr  = tmpexp;
         right->exp = tmpexp;
         
       }
@@ -247,40 +278,40 @@ static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int
 
         left->exp = expression_create( NULL, NULL, EXP_OP_STATIC, FALSE, curr_expr_id, line, first, last, FALSE );
         curr_expr_id++;
-        vector_init( left->exp->value, (vec_data*)malloc_safe( (sizeof( vec_data ) * 32), __FILE__, __LINE__ ), 32 );
-        vector_from_int( left->exp->value, left->num );
+        {
+          vector* vec = vector_create( 32, VTYPE_EXP, VDATA_UL, TRUE );
+          vector_dealloc( left->exp->value );
+          left->exp->value = vec;
+        }
+        (void)vector_from_int( left->exp->value, left->num );
 
         tmpexp = expression_create( right->exp, left->exp, op, FALSE, curr_expr_id, line, first, last, FALSE );
         curr_expr_id++;
-        right->exp->parent->expr = tmpexp;
         right->exp = tmpexp;
 
       } else {
 
         tmpexp = expression_create( right->exp, left->exp, op, FALSE, curr_expr_id, line, first, last, FALSE );
         curr_expr_id++;
-        right->exp->parent->expr = tmpexp;
-        left->exp->parent->expr  = tmpexp;
         right->exp = tmpexp;
 
       }
 
     }
 
-  } else if( op == EXP_OP_FUNC_CALL ) {
+  } else if( (op == EXP_OP_FUNC_CALL) || (op == EXP_OP_SBIT_SEL) ) {
 
     /*
-     If this is a function call, only the left expression will be a valid expression (so we need to special
+     If this is a function call or SBIT_SEL, only the left expression will be a valid expression (so we need to special
      handle this)
     */
 
     assert( right == NULL );
     assert( left  != NULL );
 
-    right = (static_expr*)malloc_safe( sizeof( static_expr ), __FILE__, __LINE__ );
+    right = (static_expr*)malloc_safe( sizeof( static_expr ) );
     right->exp = expression_create( NULL, left->exp, op, FALSE, curr_expr_id, line, first, last, FALSE );
     curr_expr_id++;
-    left->exp->parent->expr = right->exp;
 
     /* Make sure that we bind this later */
     bind_add( FUNIT_FUNCTION, func_name, right->exp, curr_funit );
@@ -302,13 +333,19 @@ static_expr* static_expr_gen( static_expr* right, static_expr* left, int op, int
 
  Calculates the LSB, width and endianness of a vector defined by the specified left and right
  static expressions.  If the width cannot be obtained immediately (parameter in static
- expression), set width to -1.  If the LSB cannot be obtained immediately (parameter in
+ expression), set width to 0.  If the LSB cannot be obtained immediately (parameter in
  static expression), set LSB to -1.  The endianness can only be used if the width is known.
  The returned width and lsb parameters can be used to size a vector instantiation.
 */
-void static_expr_calc_lsb_and_width_pre( static_expr* left, static_expr* right, int* width, int* lsb, int* big_endian ) {
+void static_expr_calc_lsb_and_width_pre(
+            static_expr*  left,
+            static_expr*  right,
+  /*@out@*/ unsigned int* width,
+  /*@out@*/ int*          lsb,
+  /*@out@*/ int*          big_endian
+) { PROFILE(STATIC_EXPR_CALC_LSB_AND_WIDTH_PRE);
 
-  *width      = -1;
+  *width      = 0;
   *lsb        = -1;
   *big_endian = 0;
 
@@ -335,6 +372,8 @@ void static_expr_calc_lsb_and_width_pre( static_expr* left, static_expr* right,
     }
   }
 
+  PROFILE_END;
+
 }
 
 /*!
@@ -348,7 +387,13 @@ void static_expr_calc_lsb_and_width_pre( static_expr* left, static_expr* right,
  static expressions.  This function assumes that any expressions have been calculated for
  a legal value.
 */
-void static_expr_calc_lsb_and_width_post( static_expr* left, static_expr* right, int* width, int* lsb, int* big_endian ) {
+void static_expr_calc_lsb_and_width_post(
+            static_expr*  left,
+            static_expr*  right,
+  /*@out@*/ unsigned int* width,
+  /*@out@*/ int*          lsb,
+  /*@out@*/ int*          big_endian
+) { PROFILE(STATIC_EXPR_CALC_LSB_AND_WIDTH_POST);
   
   assert( left  != NULL );
   assert( right != NULL );
@@ -383,6 +428,8 @@ void static_expr_calc_lsb_and_width_post( static_expr* left, static_expr* right,
     assert( *lsb >= 0 );
   }
 
+  PROFILE_END;
+
 }
 
 /*!
@@ -392,7 +439,10 @@ void static_expr_calc_lsb_and_width_post( static_expr* left, static_expr* right,
  Deallocates all allocated memory from the heap for the specified static_expr
  structure.
 */
-void static_expr_dealloc( static_expr* stexp, bool rm_exp ) {
+void static_expr_dealloc(
+  static_expr* stexp,
+  bool         rm_exp
+) { PROFILE(STATIC_EXPR_DEALLOC);
 
   if( stexp != NULL ) {
 
@@ -400,104 +450,9 @@ void static_expr_dealloc( static_expr* stexp, bool rm_exp ) {
       expression_dealloc( stexp->exp, FALSE );
     }
 
-    free_safe( stexp );
+    free_safe( stexp, sizeof( static_expr ) );
 
   }
 
 }
 
-/*
- $Log: static.c,v $
- Revision 1.17.4.1.4.1  2006/05/27 05:56:14  phase1geo
- Fixing last problem with bug fix.  Updated date on NEWS.
-
- Revision 1.17.4.1  2006/04/20 21:55:16  phase1geo
- Adding support for big endian signals.  Added new endian1 diagnostic to regression
- suite to verify this new functionality.  Full regression passes.  We may want to do
- some more testing on variants of this before calling it ready for stable release 0.4.3.
-
- Revision 1.17  2006/03/28 22:28:28  phase1geo
- Updates to user guide and added copyright information to each source file in the
- src directory.  Added test directory in user documentation directory containing the
- example used in line, toggle, combinational logic and FSM descriptions.
-
- Revision 1.16  2006/02/16 21:19:26  phase1geo
- Adding support for arrays of instances.  Also fixing some memory problems for
- constant functions and fixed binding problems when hierarchical references are
- made to merged modules.  Full regression now passes.
-
- Revision 1.15  2006/01/19 00:01:09  phase1geo
- Lots of changes/additions.  Summary report window work is now complete (with the
- exception of adding extra features).  Added support for parsing left and right
- shift operators and the exponent operator in static expression scenarios.  Fixed
- issues related to GUI (due to recent changes in the score command).  Things seem
- to be generally working as expected with the GUI now.
-
- Revision 1.14  2006/01/13 04:01:04  phase1geo
- Adding support for exponential operation.  Added exponent1 diagnostic to verify
- but Icarus does not support this currently.
-
- Revision 1.13  2005/01/07 17:59:52  phase1geo
- Finalized updates for supplemental field changes.  Everything compiles and links
- correctly at this time; however, a regression run has not confirmed the changes.
-
- Revision 1.12  2004/04/19 04:54:56  phase1geo
- Adding first and last column information to expression and related code.  This is
- not working correctly yet.
-
- Revision 1.11  2004/03/16 05:45:43  phase1geo
- Checkin contains a plethora of changes, bug fixes, enhancements...
- Some of which include:  new diagnostics to verify bug fixes found in field,
- test generator script for creating new diagnostics, enhancing error reporting
- output to include filename and line number of failing code (useful for error
- regression testing), support for error regression testing, bug fixes for
- segmentation fault errors found in field, additional data integrity features,
- and code support for GUI tool (this submission does not include TCL files).
-
- Revision 1.10  2003/11/26 23:14:41  phase1geo
- Adding code to include left-hand-side expressions of statements for report
- outputting purposes.  Full regression does not yet pass.
-
- Revision 1.9  2003/10/17 12:55:36  phase1geo
- Intermediate checkin for LSB fixes.
-
- Revision 1.8  2003/10/11 05:15:08  phase1geo
- Updates for code optimizations for vector value data type (integers to chars).
- Updated regression for changes.
-
- Revision 1.7  2002/11/02 16:16:20  phase1geo
- Cleaned up all compiler warnings in source and header files.
-
- Revision 1.6  2002/10/31 23:14:28  phase1geo
- Fixing C compatibility problems with cc and gcc.  Found a few possible problems
- with 64-bit vs. 32-bit compilation of the tool.  Fixed bug in parser that
- lead to bus errors.  Ran full regression in 64-bit mode without error.
-
- Revision 1.5  2002/10/29 19:57:51  phase1geo
- Fixing problems with beginning block comments within comments which are
- produced automatically by CVS.  Should fix warning messages from compiler.
-
- Revision 1.4  2002/10/23 03:39:07  phase1geo
- Fixing bug in MBIT_SEL expressions to calculate the expression widths
- correctly.  Updated diagnostic testsuite and added diagnostic that
- found the original bug.  A few documentation updates.
-
- Revision 1.3  2002/10/12 06:51:34  phase1geo
- Updating development documentation to match all changes within source.
- Adding new development pages created by Doxygen for the new source
- files.
-
- Revision 1.2  2002/10/11 04:24:02  phase1geo
- This checkin represents some major code renovation in the score command to
- fully accommodate parameter support.  All parameter support is in at this
- point and the most commonly used parameter usages have been verified.  Some
- bugs were fixed in handling default values of constants and expression tree
- resizing has been optimized to its fullest.  Full regression has been
- updated and passes.  Adding new diagnostics to test suite.  Fixed a few
- problems in report outputting.
-
- Revision 1.1  2002/10/02 18:55:29  phase1geo
- Adding static.c and static.h files for handling static expressions found by
- the parser.  Initial versions which compile but have not been tested.
-*/
-
