@@ -2,7 +2,7 @@
 #define __TREE_H__
 
 /*
- Copyright (c) 2006 Trevor Williams
+ Copyright (c) 2006-2009 Trevor Williams
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by the Free Software
@@ -18,7 +18,7 @@
 
 /*!
  \file     tree.h
- \author   Trevor Williams  (trevorw@charter.net)
+ \author   Trevor Williams  (phase1geo@gmail.com)
  \date     1/4/2003
  \brief    Contains functions for adding, finding, and removing nodes from binary tree.
 */
@@ -35,22 +35,7 @@ tnode* tree_find( const char* key, tnode* root );
 void tree_remove( const char* key, tnode** root );
 
 /*! \brief Deallocates entire tree from memory. */
-void tree_dealloc( tnode* root );
-
-/*
- $Log: tree.h,v $
- Revision 1.2  2006/03/28 22:28:28  phase1geo
- Updates to user guide and added copyright information to each source file in the
- src directory.  Added test directory in user documentation directory containing the
- example used in line, toggle, combinational logic and FSM descriptions.
-
- Revision 1.1  2003/01/04 09:25:15  phase1geo
- Fixing file search algorithm to fix bug where unexpected module that was
- ignored cannot be found.  Added instance7.v diagnostic to verify appropriate
- handling of this problem.  Added tree.c and tree.h and removed define_t
- structure in lexer.
-
-*/
+void tree_dealloc( /*@null@*/tnode* root );
 
 #endif
 
